
# const VTKtypemap = Dict{DataType, Int}(P1=>1, L2=>3, T3=>5,
#     Q4=>9, T4=>10, H8=>12, Q8=>23,
#     L3=>21, T6=>22,
#     T10=>24, H20=>25)
const _VTK_TYPE_MAP = Dict{AbsShapeDesc, Int}(P1=>1, L2=>3, T3=>5, Q4=>9, T4=>10, H8=>12)

"""
    vtkwrite(filename, vertices, shapes)

Write VTK file with the mesh.
"""
function vtkwrite(filename, locations, conn_increl)
    points = fill(zero(coordinatetype(locations)), nspacedims(locations), nlocations(locations))
    for i in 1:nlocations(locations)
        points[:, i] .= coordinates(locations, i)
    end #
    celltype = WriteVTK.VTKCellTypes.VTKCellType(_VTK_TYPE_MAP[shapedesc(conn_increl.left)])
    cells = [MeshCell(celltype, [j for j in conn_increl(i)]) for i in 1:nrelations(conn_increl)]
    vtkfile = vtk_grid(filename, points, cells)
    return vtk_save(vtkfile)
end

"""
    export_MESH(meshfile, mesh)

Import vertices and shapes in the MESH format.

# Output
Data dictionary, with keys
- "`vertices`" (vertices),
- "`shapes`" (array of shape collections).
"""
function export_MESH(meshfile, connectivity)
    meshfilebase, ext = splitext(meshfile)
    # Name of the shape descriptor
    t = "Unknown"
    for (k, v) in zip(keys(SHAPE_DESC), values(SHAPE_DESC))
        if  v == shapedesc(connectivity.left)
            t = k; break
        end
    end
    datinfo = [meshfilebase * "-xyz.dat", t, meshfilebase * "-conn.dat"]
    geom = attribute(connectivity.right, :geom)
    open(datinfo[1], "w") do file
        X = [geom.val(idx) for idx in 1:nlocations(geom.val.locs)] 
        writedlm(file, X, ' ')
    end
    open(datinfo[3], "w") do file
        c = [connectivity(idx) for idx in 1:nrelations(connectivity)] 
        writedlm(file, c, ' ')
    end
    open(meshfilebase * ".mesh", "w") do file
        writedlm(file, datinfo)
    end

    return true
end
