
module mmeshio1
using StaticArrays
using MeshCore: Locations, nvertices, coordinates, nshapes
using MeshPorter: import_NASTRAN
using Test
function test()
    connectivities = import_NASTRAN("trunc_cyl_shell_0.nas")
    @test length(connectivities) == 1
    connectivity = connectivities[1]
    @test (nshapes(connectivity.right), nshapes(connectivity.left)) == (376, 996)
    true
end
end
using .mmeshio1
mmeshio1.test()

module mmeshio2
using StaticArrays
using MeshCore: Locations, nvertices, coordinates, nshapes
using MeshCore: attribute, nrelations, skeleton
using MeshPorter: import_NASTRAN, vtkwrite
using Test
function test()
    connectivities = import_NASTRAN("trunc_cyl_shell_0.nas")
    @test length(connectivities) == 1
    connectivity = connectivities[1]
    @test (nshapes(connectivity.right), nshapes(connectivity.left)) == (376, 996)
    vertices = connectivity.right
    geom = attribute(vertices, :geom)
    vtkwrite("trunc_cyl_shell_0", geom.val.locs, connectivity)

    ir00 = skeleton(skeleton(skeleton(connectivity)))
    @test (nshapes(ir00.right), nshapes(ir00.left)) == (376, 376)
    vtkwrite("trunc_cyl_shell_0-0-skeleton", geom.val.locs, ir00)
    true
end
end
using .mmeshio2
mmeshio2.test()

module mmeshio5
using StaticArrays
using MeshCore: Locations, nvertices, coordinates, nshapes
using MeshCore: attribute, nrelations, skeleton
using MeshPorter: import_NASTRAN, vtkwrite
using Test
function test()
    connectivities = import_NASTRAN("trunc_cyl_shell_0.nas")
    @test length(connectivities) == 1
    connectivity = connectivities[1]
    @test (nshapes(connectivity.right), nshapes(connectivity.left)) == (376, 996)
    vertices = connectivity.right
    geom = attribute(vertices, :geom)
    vtkwrite("trunc_cyl_shell_0", geom.val.locs, connectivity)

    ir20 = skeleton(connectivity)
    @test (nshapes(ir20.right), nshapes(ir20.left)) == (376, 2368)
    vtkwrite("trunc_cyl_shell_0-2-skeleton", geom.val.locs, ir20)
    true
end
end
using .mmeshio5
mmeshio5.test()

module mmeshio6
using StaticArrays
using MeshCore: Locations, nvertices, coordinates, nshapes
using MeshCore: attribute, nrelations, skeleton
using MeshPorter: import_NASTRAN, vtkwrite
using Test
function test()
    connectivities = import_NASTRAN("trunc_cyl_shell_0.nas")
    @test length(connectivities) == 1
    connectivity = connectivities[1]
    @test (nshapes(connectivity.right), nshapes(connectivity.left)) == (376, 996)
    vertices = connectivity.right
    geom = attribute(vertices, :geom)
    vtkwrite("trunc_cyl_shell_0", geom.val.locs, connectivity)

    ir20 = skeleton(connectivity; boundaryonly = true)
    @test (nshapes(ir20.right), nshapes(ir20.left)) == (376, 752)
    vtkwrite("trunc_cyl_shell_0-boundary", geom.val.locs, ir20)
    true
end
end
using .mmeshio6
mmeshio6.test()

module mmeshio7
using StaticArrays
using MeshCore: Locations, nvertices, coordinates, nshapes
using MeshCore: attribute, nrelations, skeleton
using MeshPorter: import_NASTRAN, vtkwrite
using Test
function test()
    connectivities = import_NASTRAN("trunc_cyl_shell_0.nas")
    @test length(connectivities) == 1
    connectivity = connectivities[1]
    @test (nshapes(connectivity.right), nshapes(connectivity.left)) == (376, 996)
    vertices = connectivity.right
    geom = attribute(vertices, :geom)
    vtkwrite("trunc_cyl_shell_0", geom.val.locs, connectivity)

    ir20 = skeleton(connectivity; boundaryonly = true)
    @test (nshapes(ir20.right), nshapes(ir20.left)) == (376, 752)
    vtkwrite("trunc_cyl_shell_0-boundary", geom.val.locs, ir20)
    true
end
end
using .mmeshio7
mmeshio7.test()

module mmeshio8
using StaticArrays
using MeshCore: Locations, nvertices, coordinates, nshapes
using MeshCore: attribute, nrelations, skeleton
using MeshPorter: import_MESH, vtkwrite, export_MESH
using Test
function test()
    connectivities = import_MESH("q4-4-2.mesh")
    @test length(connectivities) == 1
    connectivity = connectivities[1]
    @test (nshapes(connectivity.right), nshapes(connectivity.left)) == (15, 8)
    vertices = connectivity.right
    geom = attribute(vertices, :geom)
    vtkwrite("q4-4-2", geom.val.locs, connectivity)

    ir20 = skeleton(connectivity; boundaryonly = true)
    @test (nshapes(ir20.right), nshapes(ir20.left)) == (15, 12)
    vtkwrite("q4-4-2-boundary", geom.val.locs, ir20)
    # @show connectivity
    @test export_MESH("q4-4-2-export.mesh", connectivity)
    true
end
end
using .mmeshio8
mmeshio8.test()

module mmeshio9
using StaticArrays
using MeshCore: Locations, nvertices, coordinates, nshapes
using MeshCore: attribute, nrelations, skeleton, nlocations
using MeshPorter: import_NASTRAN, vtkwrite, export_MESH, import_MESH
using LinearAlgebra
using Test
function test()
    connectivities = import_NASTRAN("trunc_cyl_shell_0.nas")
    connectivity = connectivities[1]
    @test (nshapes(connectivity.right), nshapes(connectivity.left)) == (376, 996)
    vertices = connectivity.right
    geom = attribute(vertices, :geom)
    
    beforeexport = (nshapes(connectivity.right), nshapes(connectivity.left))
    @test export_MESH("test.mesh", connectivity)

    connectivities = import_MESH("test.mesh")
    connectivity = connectivities[1]
    connectivity = connectivities[1]
    @test (nshapes(connectivity.right), nshapes(connectivity.left)) == beforeexport
    vertices = connectivity.right
    geom2 = attribute(vertices, :geom)
    s =  sum([norm(geom2.val(i)-geom.val(i)) for i in nlocations(geom.val.locs)] )
    @test s <= 1.0e-9
    true
end
end
using .mmeshio9
mmeshio9.test()
